import { useCallback, useMemo } from 'react';
import { useSubscription } from 'use-subscription';
import { getSnapshotPromise, identitySelector, nullSource, } from '../source';
const MissingToken = Symbol();
export const useSourceLegacy = (maybeSource, options = {}) => {
    const hasDefaultValue = 'defaultValue' in options;
    const { defaultValue } = options;
    const [core, select] = maybeSource || nullSource;
    const getCurrentValue = useCallback(() => {
        const vector = core[0]();
        return vector.length ? select(vector[0]) : MissingToken;
    }, [core, select]);
    const subscribe = core[1];
    const subscription = useMemo(() => ({
        getCurrentValue,
        subscribe,
    }), [getCurrentValue, subscribe]);
    const value = useSubscription(subscription);
    if (value === MissingToken && !hasDefaultValue) {
        throw getSnapshotPromise([core, identitySelector]);
    }
    return value === MissingToken || maybeSource === null ? defaultValue : value;
};
