import { areArraysShallowEqual } from 'retil-support';
import { observe } from './observe';
export function createStateVector(
// Doesn't accept a setter function, as it's not a hook that stores state
// between renders and thus it wouldn't make sense to do so.
initialState, 
// If provided, the state will only be updated if this function indicates
// that it is not equal to the current state.
isEqual = areArraysShallowEqual) {
    let state = initialState;
    let next = null;
    let seal = null;
    const setState = (stateOrUpdater) => {
        const nextState = typeof stateOrUpdater === 'function'
            ? stateOrUpdater(state)
            : stateOrUpdater;
        const shouldUpdate = !isEqual || !isEqual(nextState, state);
        if (shouldUpdate) {
            state = nextState;
            if (next) {
                next(state);
            }
        }
    };
    const source = observe((onNext, _, onSeal) => {
        next = onNext;
        seal = onSeal;
        onNext(state);
        return () => {
            next = null;
            seal = null;
        };
    });
    const sealState = () => {
        if (seal) {
            seal();
        }
    };
    return [source, setState, sealState];
}
